# -*- coding: utf-8 -*-

"""
    tests.controllers.controller_test_base

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import unittest
from ..http_response_catcher import HttpResponseCatcher
from raas.raas_client import RaasClient
from raas.configuration import Configuration

class ControllerTestBase(unittest.TestCase):

    """All test classes inherit from this base class. It abstracts out
    common functionality and configuration variables set up."""

    @classmethod
    def setUpClass(cls):
        """Class method called once before running tests in a test class."""
        cls.api_client = RaasClient()
        cls.request_timeout = 30
        cls.assert_precision = 0.01

        # Set Configuration parameters for test execution
        Configuration.environment = Configuration.Environment.SANDBOX


    def setUp(self):
        """Method called once before every test in a test class."""
        self.response_catcher = HttpResponseCatcher()
        self.controller.http_call_back =  self.response_catcher

    


# -*- coding: utf-8 -*-

"""
    raas.models.create_order_request_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.name_email_model

class CreateOrderRequestModel(object):

    """Implementation of the 'CreateOrderRequest' model.

    Represents the request to place an order

    Attributes:
        account_identifier (string): The account identifier
        amount (float): The order amount
        customer_identifier (string): The customer identifier
        send_email (bool): Indicates whether we should deliver this reward via
            email
        utid (string): The UTID
        campaign (string): An optional campaign identifier
        email_subject (string): The subject of the gift email
        external_ref_id (string): An optional external reference id
        message (string): The gift message in the email
        recipient (NameEmailModel): The recipient's information
        sender (NameEmailModel): Optional sender information
        notes (string): Optional notes (not displayed to customer)
        etid (string): The email template identifier

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "account_identifier":'accountIdentifier',
        "amount":'amount',
        "customer_identifier":'customerIdentifier',
        "send_email":'sendEmail',
        "utid":'utid',
        "campaign":'campaign',
        "email_subject":'emailSubject',
        "external_ref_id":'externalRefID',
        "message":'message',
        "recipient":'recipient',
        "sender":'sender',
        "notes":'notes',
        "etid":'etid'
    }

    def __init__(self,
                 account_identifier=None,
                 amount=None,
                 customer_identifier=None,
                 send_email=None,
                 utid=None,
                 campaign=None,
                 email_subject=None,
                 external_ref_id=None,
                 message=None,
                 recipient=None,
                 sender=None,
                 notes=None,
                 etid=None):
        """Constructor for the CreateOrderRequestModel class"""

        # Initialize members of the class
        self.account_identifier = account_identifier
        self.amount = amount
        self.customer_identifier = customer_identifier
        self.send_email = send_email
        self.utid = utid
        self.campaign = campaign
        self.email_subject = email_subject
        self.external_ref_id = external_ref_id
        self.message = message
        self.recipient = recipient
        self.sender = sender
        self.notes = notes
        self.etid = etid


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        account_identifier = dictionary.get('accountIdentifier')
        amount = dictionary.get('amount')
        customer_identifier = dictionary.get('customerIdentifier')
        send_email = dictionary.get('sendEmail')
        utid = dictionary.get('utid')
        campaign = dictionary.get('campaign')
        email_subject = dictionary.get('emailSubject')
        external_ref_id = dictionary.get('externalRefID')
        message = dictionary.get('message')
        recipient = raas.models.name_email_model.NameEmailModel.from_dictionary(dictionary.get('recipient')) if dictionary.get('recipient') else None
        sender = raas.models.name_email_model.NameEmailModel.from_dictionary(dictionary.get('sender')) if dictionary.get('sender') else None
        notes = dictionary.get('notes')
        etid = dictionary.get('etid')

        # Return an object of this model
        return cls(account_identifier,
                   amount,
                   customer_identifier,
                   send_email,
                   utid,
                   campaign,
                   email_subject,
                   external_ref_id,
                   message,
                   recipient,
                   sender,
                   notes,
                   etid)





# -*- coding: utf-8 -*-

"""
    raas.models.brand_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper
import raas.models.item_model

class BrandModel(object):

    """Implementation of the 'Brand' model.

    Represents a Brand

    Attributes:
        brand_key (string): The brand key
        brand_name (string): The brand name
        disclaimer (string): The brand's disclaimer
        description (string): The brand's description
        short_description (string): The brand's short description
        terms (string): The brand's terms
        created_date (datetime): The date the brand was created
        last_update_date (datetime): The date the brand was last updated
        image_urls (dict<object, string>): A map of Image URLs
        status (string): The brand's status
        items (list of ItemModel): An array of Item objects

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_key":'brandKey',
        "brand_name":'brandName',
        "disclaimer":'disclaimer',
        "description":'description',
        "short_description":'shortDescription',
        "terms":'terms',
        "created_date":'createdDate',
        "last_update_date":'lastUpdateDate',
        "image_urls":'imageUrls',
        "status":'status',
        "items":'items'
    }

    def __init__(self,
                 brand_key=None,
                 brand_name=None,
                 disclaimer=None,
                 description=None,
                 short_description=None,
                 terms=None,
                 created_date=None,
                 last_update_date=None,
                 image_urls=None,
                 status=None,
                 items=None):
        """Constructor for the BrandModel class"""

        # Initialize members of the class
        self.brand_key = brand_key
        self.brand_name = brand_name
        self.disclaimer = disclaimer
        self.description = description
        self.short_description = short_description
        self.terms = terms
        self.created_date = APIHelper.RFC3339DateTime(created_date) if created_date else None
        self.last_update_date = APIHelper.RFC3339DateTime(last_update_date) if last_update_date else None
        self.image_urls = image_urls
        self.status = status
        self.items = items


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_key = dictionary.get('brandKey')
        brand_name = dictionary.get('brandName')
        disclaimer = dictionary.get('disclaimer')
        description = dictionary.get('description')
        short_description = dictionary.get('shortDescription')
        terms = dictionary.get('terms')
        created_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdDate")).datetime if dictionary.get("createdDate") else None
        last_update_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("lastUpdateDate")).datetime if dictionary.get("lastUpdateDate") else None
        image_urls = dictionary.get('imageUrls')
        status = dictionary.get('status')
        items = None
        if dictionary.get('items') != None:
            items = list()
            for structure in dictionary.get('items'):
                items.append(raas.models.item_model.ItemModel.from_dictionary(structure))

        # Return an object of this model
        return cls(brand_key,
                   brand_name,
                   disclaimer,
                   description,
                   short_description,
                   terms,
                   created_date,
                   last_update_date,
                   image_urls,
                   status,
                   items)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_customers_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper


class CustomersControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(CustomersControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.customers

    # Tests retrieving all customers
    def test_test_get_all_customers(self):

        # Perform the API call through the SDK function
        result = self.controller.get_all_customers()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"customerIdentifier":"sdkautotest1","displayName":"SDK Auto Testing 1","'
            'status":"active","createdAt":"2018-04-26T18:13:12.874Z","accounts":[{"accou'
            'ntIdentifier":"sdkautotest3","accountNumber":"A01335766","displayName":"SDK'
            ' Auto Testing 3","createdAt":"2018-04-26T18:16:51.652Z","status":"ACTIVE"},'
            '{"accountIdentifier":"sdkautotest2","accountNumber":"A11720237","displayNam'
            'e":"SDK Auto Testing 2","createdAt":"2018-04-26T18:13:45.196Z","status":"AC'
            'TIVE"}]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests retrieving a single customer
    def test_test_get_customer(self):
        # Parameters for the API call
        customer_identifier = 'sdkautotest1'

        # Perform the API call through the SDK function
        result = self.controller.get_customer(customer_identifier)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"customerIdentifier":"sdkautotest1","displayName":"SDK Auto Testing 1","s'
            'tatus":"active","createdAt":"2018-04-26T18:13:12.874Z","accounts":[{"accoun'
            'tIdentifier":"sdkautotest3","accountNumber":"A01335766","displayName":"SDK '
            'Auto Testing 3","createdAt":"2018-04-26T18:16:51.652Z","status":"ACTIVE"},{'
            '"accountIdentifier":"sdkautotest2","accountNumber":"A11720237","displayName'
            '":"SDK Auto Testing 2","createdAt":"2018-04-26T18:13:45.196Z","status":"ACT'
            'IVE"}]}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))





# -*- coding: utf-8 -*-

"""
    raas.models.name_email_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class NameEmailModel(object):

    """Implementation of the 'NameEmail' model.

    Represents a name and an email address

    Attributes:
        email (string): An email address
        first_name (string): A first name
        last_name (string): A last name

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "email":'email',
        "first_name":'firstName',
        "last_name":'lastName'
    }

    def __init__(self,
                 email=None,
                 first_name=None,
                 last_name=None):
        """Constructor for the NameEmailModel class"""

        # Initialize members of the class
        self.email = email
        self.first_name = first_name
        self.last_name = last_name


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        email = dictionary.get('email')
        first_name = dictionary.get('firstName')
        last_name = dictionary.get('lastName')

        # Return an object of this model
        return cls(email,
                   first_name,
                   last_name)





# -*- coding: utf-8 -*-

"""
    raas.controllers.fund_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.unregister_credit_card_response_model import UnregisterCreditCardResponseModel
from ..models.get_deposit_response_model import GetDepositResponseModel
from ..models.deposit_response_model import DepositResponseModel
from ..models.credit_card_model import CreditCardModel
from ..exceptions.raas_generic_exception import RaasGenericException

class FundController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(FundController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def create_unregister_credit_card(self,
                                      body):
        """Does a POST request to /creditCardUnregisters.

        Unregister a credit card

        Args:
            body (UnregisterCreditCardRequestModel): TODO: type description
                here. Example: 

        Returns:
            UnregisterCreditCardResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_unregister_credit_card called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_unregister_credit_card.')
            self.validate_parameters(body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_unregister_credit_card.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCardUnregisters'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_unregister_credit_card.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_unregister_credit_card.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_unregister_credit_card')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_unregister_credit_card.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, UnregisterCreditCardResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_deposit(self,
                    deposit_id):
        """Does a GET request to /creditCardDeposits/{depositId}.

        Get details for a specific credit card deposit

        Args:
            deposit_id (string): The reference deposit id

        Returns:
            GetDepositResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_deposit called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_deposit.')
            self.validate_parameters(deposit_id=deposit_id)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_deposit.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCardDeposits/{depositId}'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'depositId': deposit_id
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_deposit.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_deposit.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_deposit')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_deposit.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, GetDepositResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def add_funds(self,
                  body):
        """Does a POST request to /creditCardDeposits.

        Funds an account via credit card

        Args:
            body (DepositRequestModel): TODO: type description here. Example:
                
        Returns:
            DepositResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('add_funds called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for add_funds.')
            self.validate_parameters(body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for add_funds.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCardDeposits'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for add_funds.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for add_funds.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'add_funds')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for add_funds.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, DepositResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_credit_cards(self):
        """Does a GET request to /creditCards.

        Retrieves all credit cards registered on the platform

        Returns:
            list of CreditCardModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_credit_cards called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_credit_cards.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCards'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_credit_cards.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_credit_cards.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_credit_cards')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_credit_cards.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CreditCardModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_credit_card(self,
                        token):
        """Does a GET request to /creditCards/{token}.

        Retrieves details for a single credit card

        Args:
            token (string): Credit Card Token

        Returns:
            CreditCardModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_credit_card called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_credit_card.')
            self.validate_parameters(token=token)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_credit_card.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCards/{token}'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'token': token
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_credit_card.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_credit_card.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_credit_card')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_credit_card.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CreditCardModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def create_register_credit_card(self,
                                    body):
        """Does a POST request to /creditCards.

        Registers a new credit card

        Args:
            body (CreateCreditCardRequestModel): A CreateCreditCardRequest
                object

        Returns:
            CreditCardModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_register_credit_card called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_register_credit_card.')
            self.validate_parameters(body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_register_credit_card.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/creditCards'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_register_credit_card.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_register_credit_card.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_register_credit_card')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_register_credit_card.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CreditCardModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    raas.models.raas_client_exception

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from ..api_helper import APIHelper
import raas.exceptions.api_exception
import raas.models.raas_client_error_model

class RaasClientException(raas.exceptions.api_exception.APIException):
    def __init__(self, reason, context):
        """Constructor for the RaasClientException class

        Args:
            reason (string): The reason (or error message) for the Exception
                to be raised.
            context (HttpContext):  The HttpContext of the API call.

        """
        super(RaasClientException, self).__init__(reason, context)
        dictionary = APIHelper.json_deserialize(self.context.response.raw_body)
        if isinstance(dictionary, dict):
            self.unbox(dictionary)

    def unbox(self, dictionary):
        """Populates the properties of this object by extracting them from a dictionary.

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        """
        self.timestamp = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestamp")).datetime if dictionary.get("timestamp") else None
        self.request_id = dictionary.get('requestId')
        self.path = dictionary.get('path')
        self.http_code = dictionary.get('httpCode')
        self.http_phrase = dictionary.get('httpPhrase')
        self.errors = None
        if dictionary.get('errors') != None:
            self.errors = list()
            for structure in dictionary.get('errors'):
                self.errors.append(raas.models.raas_client_error_model.RaasClientErrorModel.from_dictionary(structure))



# -*- coding: utf-8 -*-

"""
    raas.models.exchange_rate_response_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.exchange_rate_model

class ExchangeRateResponseModel(object):

    """Implementation of the 'ExchangeRateResponse' model.

    Represents the response from the get exchange rates call

    Attributes:
        disclaimer (string): A disclaimer about the exchange rates returned
        exchange_rates (list of ExchangeRateModel): An array of ExchangeRate
            objects

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "disclaimer":'disclaimer',
        "exchange_rates":'exchangeRates'
    }

    def __init__(self,
                 disclaimer=None,
                 exchange_rates=None):
        """Constructor for the ExchangeRateResponseModel class"""

        # Initialize members of the class
        self.disclaimer = disclaimer
        self.exchange_rates = exchange_rates


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        disclaimer = dictionary.get('disclaimer')
        exchange_rates = None
        if dictionary.get('exchangeRates') != None:
            exchange_rates = list()
            for structure in dictionary.get('exchangeRates'):
                exchange_rates.append(raas.models.exchange_rate_model.ExchangeRateModel.from_dictionary(structure))

        # Return an object of this model
        return cls(disclaimer,
                   exchange_rates)





# -*- coding: utf-8 -*-

"""
    raas.controllers.customers_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.customer_model import CustomerModel
from ..exceptions.raas_generic_exception import RaasGenericException

class CustomersController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(CustomersController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def create_customer(self,
                        body):
        """Does a POST request to /customers.

        Creates a new customer

        Args:
            body (CreateCustomerRequestModel): Request Body

        Returns:
            CustomerModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_customer called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_customer.')
            self.validate_parameters(body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_customer.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/customers'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_customer.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_customer.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_customer')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_customer.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CustomerModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_all_customers(self):
        """Does a GET request to /customers.

        Retrieves all customers under the platform

        Returns:
            list of CustomerModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_all_customers called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_all_customers.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/customers'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_all_customers.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_all_customers.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_all_customers')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_all_customers.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CustomerModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_customer(self,
                     customer_identifier):
        """Does a GET request to /customers/{customerIdentifier}.

        Retrieves a single customer

        Args:
            customer_identifier (string): Customer Identifier

        Returns:
            CustomerModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_customer called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_customer.')
            self.validate_parameters(customer_identifier=customer_identifier)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_customer.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/customers/{customerIdentifier}'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'customerIdentifier': customer_identifier
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_customer.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_customer.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_customer')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_customer.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CustomerModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
   raas.configuration

   This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import sys
import logging

from .api_helper import APIHelper

logging.basicConfig(stream=sys.stdout, level=logging.INFO)

class Configuration(object):

    """A class used for configuring the SDK by a user.

    This class need not be instantiated and all properties and methods
    are accessible without instance creation.

    """

    # Set the array parameter serialization method
    # (allowed: indexed, unindexed, plain, csv, tsv, psv)
    array_serialization = "indexed"

    # An enum for SDK environments
    class Environment(object):
        # The sandbox environment does not use real money and can be used for testing.
        SANDBOX = 0
        # The production environment uses real money and is for live transactions.
        PRODUCTION = 1

    # An enum for API servers
    class Server(object):
        DEFAULT = 0

    # The environment in which the SDK is running
    environment = Environment.SANDBOX

    # Platform Name (Provided by Tango Card)
    platform_name = 'QAPlatform2'

    # Platform Key (Provided by Tango Card)
    platform_key = 'apYPfT6HNONpDRUj3CLGWYt7gvIHONpDRUYPfT6Hj'

    # All the environments the SDK can run in
    environments = {
        Environment.SANDBOX: {
            Server.DEFAULT: 'https://integration-api.tangocard.com/raas/v2',
        },
        Environment.PRODUCTION: {
            Server.DEFAULT: 'https://api.tangocard.com/raas/v2',
        },
    }

    @classmethod
    def get_base_uri(cls, server=Server.DEFAULT):
        """Generates the appropriate base URI for the environment and the server.

        Args:
            server (Configuration.Server): The server enum for which the base URI is required.

        Returns:
            String: The base URI.

        """
        return cls.environments[cls.environment][server]



# -*- coding: utf-8 -*-

"""
    raas.models.deposit_request_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class DepositRequestModel(object):

    """Implementation of the 'DepositRequest' model.

    Represents the request to deposit funds via credit card

    Attributes:
        customer_identifier (string): The customer identifier
        account_identifier (string): The account identifier
        credit_card_token (string): The credit card token
        amount (float): The amount to fund

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "customer_identifier":'customerIdentifier',
        "account_identifier":'accountIdentifier',
        "credit_card_token":'creditCardToken',
        "amount":'amount'
    }

    def __init__(self,
                 customer_identifier=None,
                 account_identifier=None,
                 credit_card_token=None,
                 amount=None):
        """Constructor for the DepositRequestModel class"""

        # Initialize members of the class
        self.customer_identifier = customer_identifier
        self.account_identifier = account_identifier
        self.credit_card_token = credit_card_token
        self.amount = amount


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        customer_identifier = dictionary.get('customerIdentifier')
        account_identifier = dictionary.get('accountIdentifier')
        credit_card_token = dictionary.get('creditCardToken')
        amount = dictionary.get('amount')

        # Return an object of this model
        return cls(customer_identifier,
                   account_identifier,
                   credit_card_token,
                   amount)





# -*- coding: utf-8 -*-

"""
    raas.models.catalog_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.brand_model

class CatalogModel(object):

    """Implementation of the 'Catalog' model.

    Represents a Catalog

    Attributes:
        catalog_name (string): The name of your catalog
        brands (list of BrandModel): The brands in your catalog

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "catalog_name":'catalogName',
        "brands":'brands'
    }

    def __init__(self,
                 catalog_name=None,
                 brands=None):
        """Constructor for the CatalogModel class"""

        # Initialize members of the class
        self.catalog_name = catalog_name
        self.brands = brands


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        catalog_name = dictionary.get('catalogName')
        brands = None
        if dictionary.get('brands') != None:
            brands = list()
            for structure in dictionary.get('brands'):
                brands.append(raas.models.brand_model.BrandModel.from_dictionary(structure))

        # Return an object of this model
        return cls(catalog_name,
                   brands)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_fund_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper
from raas.models.deposit_request_model import DepositRequestModel


class FundControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(FundControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.fund

    # Tests retrieving deposit information
    def test_test_get_deposit(self):
        # Parameters for the API call
        deposit_id = 'RAD-180426-5407'

        # Perform the API call through the SDK function
        result = self.controller.get_deposit(deposit_id)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"referenceDepositID":"RAD-180426-5407","amount":1.25,"amountCharged":1.29'
            ',"feePercent":3.5,"createdDate":"2018-04-26T18:56:28.28Z","status":"SUCCESS'
            '","accountNumber":"A32386768"}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests adding funds to an account
    def test_test_add_funds(self):
        # Parameters for the API call
        body = APIHelper.json_deserialize(
            '{"accountIdentifier":"sdkautotest5","amount":1.25,"creditCardToken":"56ac1'
            'a30-6ba2-4047-9b8c-70f97a5502c5","customerIdentifier":"sdkautotest4"}'
            , DepositRequestModel.from_dictionary)

        # Perform the API call through the SDK function
        result = self.controller.add_funds(body)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"referenceDepositID":"RAD-180426-5407","amount":1.25,"amountCharged":1.29'
            ',"feePercent":3.5,"createdDate":"2018-04-26T18:56:28.28Z","status":"SUCCESS'
            '","accountNumber":"A32386768"}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Tests retrieving all credit cards for a platform
    def test_test_get_credit_cards(self):

        # Perform the API call through the SDK function
        result = self.controller.get_credit_cards()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"customerIdentifier":"sdkautotest4","accountIdentifier":"sdkautotest5","'
            'token":"56ac1a30-6ba2-4047-9b8c-70f97a5502c5","label":"SDK Auto Testing Car'
            'd 1","lastFourDigits":"4444","expirationDate":"2020-01","status":"ACTIVE","'
            'createdDate":"2018-04-26T18:52:17.575Z","activationDate":"2018-04-26T18:52:'
            '17.575Z","contactInformation":[{"fullName":"Test User","emailAddress":"test'
            '@example.com"},{"fullName":"Test Man Two","emailAddress":"test2@example.com'
            '"}],"accountNumber":"A32386768"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests retrieving a single credit card
    def test_test_get_credit_card(self):
        # Parameters for the API call
        token = '56ac1a30-6ba2-4047-9b8c-70f97a5502c5'

        # Perform the API call through the SDK function
        result = self.controller.get_credit_card(token)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"customerIdentifier":"sdkautotest4","accountIdentifier":"sdkautotest5","t'
            'oken":"56ac1a30-6ba2-4047-9b8c-70f97a5502c5","label":"SDK Auto Testing Card'
            ' 1","lastFourDigits":"4444","expirationDate":"2020-01","status":"ACTIVE","c'
            'reatedDate":"2018-04-26T18:52:17.575Z","activationDate":"2018-04-26T18:52:1'
            '7.575Z","contactInformation":[{"fullName":"Test User","emailAddress":"test@'
            'example.com"},{"fullName":"Test Man Two","emailAddress":"test2@example.com"'
            '}],"accountNumber":"A32386768"}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))





# -*- coding: utf-8 -*-

"""
    raas.controllers.accounts_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.account_summary_model import AccountSummaryModel
from ..models.account_model import AccountModel
from ..exceptions.raas_generic_exception import RaasGenericException

class AccountsController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(AccountsController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def get_accounts_by_customer(self,
                                 customer_identifier):
        """Does a GET request to /customers/{customerIdentifier}/accounts.

        Retrieves a list of accounts for a given customer

        Args:
            customer_identifier (string): Customer Identifier

        Returns:
            list of AccountSummaryModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_accounts_by_customer called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_accounts_by_customer.')
            self.validate_parameters(customer_identifier=customer_identifier)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_accounts_by_customer.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/customers/{customerIdentifier}/accounts'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'customerIdentifier': customer_identifier
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_accounts_by_customer.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_accounts_by_customer.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_accounts_by_customer')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_accounts_by_customer.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, AccountSummaryModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def create_account(self,
                       customer_identifier,
                       body):
        """Does a POST request to /customers/{customerIdentifier}/accounts.

        Creates an account under a given customer

        Args:
            customer_identifier (string): Customer Identifier
            body (CreateAccountRequestModel): Request Body

        Returns:
            AccountModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_account called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_account.')
            self.validate_parameters(customer_identifier=customer_identifier,
                                     body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_account.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/customers/{customerIdentifier}/accounts'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'customerIdentifier': customer_identifier
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_account.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_account.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_account')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_account.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, AccountModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_all_accounts(self):
        """Does a GET request to /accounts.

        Retrieves all accounts under the platform

        Returns:
            list of AccountModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_all_accounts called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_all_accounts.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/accounts'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_all_accounts.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_all_accounts.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_all_accounts')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_all_accounts.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, AccountModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_account(self,
                    account_identifier):
        """Does a GET request to /accounts/{accountIdentifier}.

        Retrieves a single account

        Args:
            account_identifier (string): Account Identifier

        Returns:
            AccountModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_account called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_account.')
            self.validate_parameters(account_identifier=account_identifier)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_account.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/accounts/{accountIdentifier}'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'accountIdentifier': account_identifier
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_account.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_account.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_account')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_account.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, AccountModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    raas.models.unregister_credit_card_response_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class UnregisterCreditCardResponseModel(object):

    """Implementation of the 'UnregisterCreditCardResponse' model.

    Represents the response from the unregister credit card call

    Attributes:
        created_date (datetime): The date the card was removed
        message (string): A message describing the status of the card
        token (string): The credit card token

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "created_date":'createdDate',
        "message":'message',
        "token":'token'
    }

    def __init__(self,
                 created_date=None,
                 message=None,
                 token=None):
        """Constructor for the UnregisterCreditCardResponseModel class"""

        # Initialize members of the class
        self.created_date = APIHelper.RFC3339DateTime(created_date) if created_date else None
        self.message = message
        self.token = token


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        created_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdDate")).datetime if dictionary.get("createdDate") else None
        message = dictionary.get('message')
        token = dictionary.get('token')

        # Return an object of this model
        return cls(created_date,
                   message,
                   token)





# -*- coding: utf-8 -*-

"""
    raas.models.new_credit_card_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class NewCreditCardModel(object):

    """Implementation of the 'NewCreditCard' model.

    Represents the credit card information required to register a credit card

    Attributes:
        number (string): The credit card number
        expiration (string): The credit card expiration date in YYYY-MM
            format
        verification_number (string): The 3 or 4 digit card security code on
            the back of card

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "number":'number',
        "expiration":'expiration',
        "verification_number":'verificationNumber'
    }

    def __init__(self,
                 number=None,
                 expiration=None,
                 verification_number=None):
        """Constructor for the NewCreditCardModel class"""

        # Initialize members of the class
        self.number = number
        self.expiration = expiration
        self.verification_number = verification_number


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        number = dictionary.get('number')
        expiration = dictionary.get('expiration')
        verification_number = dictionary.get('verificationNumber')

        # Return an object of this model
        return cls(number,
                   expiration,
                   verification_number)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_catalog_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper


class CatalogControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(CatalogControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.catalog

    # Tests if we can successfully retrieve a platform's catalog
    def test_test_retrieve_catalog(self):

        # Perform the API call through the SDK function
        result = self.controller.get_catalog()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"catalogName":"TestCatalog","brands":[{"brandKey":"B916708","brandName":"'
            'Amazon.com","disclaimer":"disclaimer","description":"desc","shortDescriptio'
            'n":"short desc","terms":"terms","createdDate":"2016-06-17T16:52:24Z","lastU'
            'pdateDate":"2017-10-23T22:18:51Z","imageUrls":{"80w-326ppi":"https://d30s7y'
            'zk2az89n.cloudfront.net/images/brands/b916708-80w-326ppi.png","130w-326ppi"'
            ':"https://d30s7yzk2az89n.cloudfront.net/images/brands/b916708-130w-326ppi.p'
            'ng","200w-326ppi":"https://d30s7yzk2az89n.cloudfront.net/images/brands/b916'
            '708-200w-326ppi.png","278w-326ppi":"https://d30s7yzk2az89n.cloudfront.net/i'
            'mages/brands/b916708-278w-326ppi.png","300w-326ppi":"https://d30s7yzk2az89n'
            '.cloudfront.net/images/brands/b916708-300w-326ppi.png","1200w-326ppi":"http'
            's://d30s7yzk2az89n.cloudfront.net/images/brands/b916708-1200w-326ppi.png"},'
            '"status":"active","items":[{"utid":"U666425","rewardName":"Amazon.com Gift '
            'Card","currencyCode":"USD","status":"active","valueType":"VARIABLE_VALUE","'
            'rewardType":"gift card","minValue":0.01,"maxValue":2000,"createdDate":"2016'
            '-06-17T17:38:45.294Z","lastUpdateDate":"2017-12-15T01:27:49.607Z","countrie'
            's":["US"]}]},{"brandKey":"B725361","brandName":"AMC Theatres®","disclaimer"'
            ':"discl","description":"desc","shortDescription":"short desc","terms":"term'
            's","createdDate":"2016-06-23T21:53:45Z","lastUpdateDate":"2016-07-25T22:41:'
            '11Z","imageUrls":{"80w-326ppi":"https://d30s7yzk2az89n.cloudfront.net/image'
            's/brands/b725361-80w-326ppi.png","130w-326ppi":"https://d30s7yzk2az89n.clou'
            'dfront.net/images/brands/b725361-130w-326ppi.png","200w-326ppi":"https://d3'
            '0s7yzk2az89n.cloudfront.net/images/brands/b725361-200w-326ppi.png","278w-32'
            '6ppi":"https://d30s7yzk2az89n.cloudfront.net/images/brands/b725361-278w-326'
            'ppi.png","300w-326ppi":"https://d30s7yzk2az89n.cloudfront.net/images/brands'
            '/b725361-300w-326ppi.png","1200w-326ppi":"https://d30s7yzk2az89n.cloudfront'
            '.net/images/brands/b725361-1200w-326ppi.png"},"status":"active","items":[{"'
            'utid":"U154092","rewardName":"AMC® Gift Card $10.00","currencyCode":"USD","'
            'status":"active","valueType":"FIXED_VALUE","rewardType":"gift card","faceVa'
            'lue":10,"createdDate":"2016-07-27T02:54:30.142Z","lastUpdateDate":"2016-09-'
            '21T20:59:01.874Z","countries":["US"]},{"utid":"U913336","rewardName":"AMC® '
            'Gift Card $25.00","currencyCode":"USD","status":"active","valueType":"FIXED'
            '_VALUE","rewardType":"gift card","faceValue":25,"createdDate":"2016-07-27T0'
            '2:55:37.899Z","lastUpdateDate":"2016-10-17T20:55:00.386Z","countries":["US"'
            ']},{"utid":"U652361","rewardName":"AMC® Gift Card $50.00","currencyCode":"U'
            'SD","status":"active","valueType":"FIXED_VALUE","rewardType":"gift card","f'
            'aceValue":50,"createdDate":"2016-07-27T02:56:53.391Z","lastUpdateDate":"201'
            '6-09-21T20:59:16.283Z","countries":["US"]}]}]}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))





# -*- coding: utf-8 -*-

"""
    raas.models.currency_breakdown_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class CurrencyBreakdownModel(object):

    """Implementation of the 'CurrencyBreakdown' model.

    Represents a breakdown of an amount and currency information

    Attributes:
        currency_code (string): A 3 digit currency code
        exchange_rate (float): The exchange rate
        fee (float): The amount of the fee applied
        total (float): The order total
        value (float): The value of the order

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currency_code":'currencyCode',
        "exchange_rate":'exchangeRate',
        "fee":'fee',
        "total":'total',
        "value":'value'
    }

    def __init__(self,
                 currency_code=None,
                 exchange_rate=None,
                 fee=None,
                 total=None,
                 value=None):
        """Constructor for the CurrencyBreakdownModel class"""

        # Initialize members of the class
        self.currency_code = currency_code
        self.exchange_rate = exchange_rate
        self.fee = fee
        self.total = total
        self.value = value


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        currency_code = dictionary.get('currencyCode')
        exchange_rate = dictionary.get('exchangeRate')
        fee = dictionary.get('fee')
        total = dictionary.get('total')
        value = dictionary.get('value')

        # Return an object of this model
        return cls(currency_code,
                   exchange_rate,
                   fee,
                   total,
                   value)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_accounts_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper


class AccountsControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(AccountsControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.accounts

    # Tests retrieving all accounts under a specific customer
    def test_test_get_all_customer_accounts(self):
        # Parameters for the API call
        customer_identifier = 'sdkautotest1'

        # Perform the API call through the SDK function
        result = self.controller.get_accounts_by_customer(customer_identifier)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"accountIdentifier":"sdkautotest3","accountNumber":"A01335766","displayN'
            'ame":"SDK Auto Testing 3","createdAt":"2018-04-26T18:16:51.652Z","status":"'
            'ACTIVE"},{"accountIdentifier":"sdkautotest2","accountNumber":"A11720237","d'
            'isplayName":"SDK Auto Testing 2","createdAt":"2018-04-26T18:13:45.196Z","st'
            'atus":"ACTIVE"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests retrieving all accounts
    def test_test_get_all_accounts(self):

        # Perform the API call through the SDK function
        result = self.controller.get_all_accounts()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"accountIdentifier":"sdkautotest2","accountNumber":"A11720237","displayN'
            'ame":"SDK Auto Testing 2","currencyCode":"USD","currentBalance":0,"createdA'
            't":"2018-04-26T18:13:45.196Z","status":"ACTIVE","contactEmail":"test@exampl'
            'e.com"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests retrieving a single account
    def test_test_get_single_account(self):
        # Parameters for the API call
        account_identifier = 'sdkautotest2'

        # Perform the API call through the SDK function
        result = self.controller.get_account(account_identifier)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"accountIdentifier":"sdkautotest2","accountNumber":"A11720237","displayNa'
            'me":"SDK Auto Testing 2","currencyCode":"USD","currentBalance":0,"createdAt'
            '":"2018-04-26T18:13:45.196Z","status":"ACTIVE","contactEmail":"test@example'
            '.com"}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))





# -*- coding: utf-8 -*-

"""
    raas.controllers.orders_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.order_model import OrderModel
from ..models.resend_order_response_model import ResendOrderResponseModel
from ..models.get_orders_response_model import GetOrdersResponseModel
from ..exceptions.raas_generic_exception import RaasGenericException

class OrdersController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(OrdersController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def get_order(self,
                  reference_order_id):
        """Does a GET request to /orders/{referenceOrderID}.

        Retrieves a single order

        Args:
            reference_order_id (string): Reference Order ID

        Returns:
            OrderModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_order called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for get_order.')
            self.validate_parameters(reference_order_id=reference_order_id)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_order.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/orders/{referenceOrderID}'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'referenceOrderID': reference_order_id
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_order.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_order.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_order')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_order.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, OrderModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def create_resend_order(self,
                            reference_order_id):
        """Does a POST request to /orders/{referenceOrderID}/resends.

        Resends an order

        Args:
            reference_order_id (string): The order's reference order id

        Returns:
            ResendOrderResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_resend_order called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_resend_order.')
            self.validate_parameters(reference_order_id=reference_order_id)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_resend_order.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/orders/{referenceOrderID}/resends'
            _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
                'referenceOrderID': reference_order_id
            })
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_resend_order.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_resend_order.')
            _request = self.http_client.post(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_resend_order')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_resend_order.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, ResendOrderResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def get_orders(self,
                   options=dict()):
        """Does a GET request to /orders.

        Retrieves a list of orders under a platform

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    account_identifier -- string -- Account identifier
                    customer_identifier -- string -- Customer identifier
                    external_ref_id -- string -- External reference id
                    start_date -- datetime -- The start date
                    end_date -- datetime -- The end date
                    elements_per_block -- int -- The number of elements per
                        page
                    page -- int -- The page number to return

        Returns:
            GetOrdersResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_orders called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_orders.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/orders'
            _query_parameters = {
                'accountIdentifier': options.get('account_identifier', None),
                'customerIdentifier': options.get('customer_identifier', None),
                'externalRefID': options.get('external_ref_id', None),
                'startDate': APIHelper.when_defined(APIHelper.RFC3339DateTime, options.get('start_date', None)),
                'endDate': APIHelper.when_defined(APIHelper.RFC3339DateTime, options.get('end_date', None)),
                'elementsPerBlock': options.get('elements_per_block', None),
                'page': options.get('page', None)
            }
            _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
                _query_parameters, Configuration.array_serialization)
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_orders.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_orders.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_orders')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_orders.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, GetOrdersResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise

    def create_order(self,
                     body):
        """Does a POST request to /orders.

        Places an order

        Args:
            body (CreateOrderRequestModel): A CreateOrderRequest object

        Returns:
            OrderModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('create_order called.')
    
            # Validate required parameters
            self.logger.info('Validating required parameters for create_order.')
            self.validate_parameters(body=body)
    
            # Prepare query URL
            self.logger.info('Preparing query URL for create_order.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/orders'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for create_order.')
            _headers = {
                'accept': 'application/json',
                'content-type': 'application/json; charset=utf-8'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for create_order.')
            _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'create_order')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for create_order.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, OrderModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    raas.models.create_credit_card_request_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.new_credit_card_model
import raas.models.billing_address_model
import raas.models.full_name_email_model

class CreateCreditCardRequestModel(object):

    """Implementation of the 'CreateCreditCardRequest' model.

    Represents the request to register a credit card

    Attributes:
        customer_identifier (string): The customer identifier
        account_identifier (string): The account identifier
        label (string): The credit card's label/nickname
        ip_address (string): The IP address of the user registering the card
        credit_card (NewCreditCardModel): A NewCreditCard object
        billing_address (BillingAddressModel): A BillingAddress object
        contact_information (list of FullNameEmailModel): An optional array of
            FullNameEmail objects

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "customer_identifier":'customerIdentifier',
        "account_identifier":'accountIdentifier',
        "label":'label',
        "ip_address":'ipAddress',
        "credit_card":'creditCard',
        "billing_address":'billingAddress',
        "contact_information":'contactInformation'
    }

    def __init__(self,
                 customer_identifier=None,
                 account_identifier=None,
                 label=None,
                 ip_address=None,
                 credit_card=None,
                 billing_address=None,
                 contact_information=None):
        """Constructor for the CreateCreditCardRequestModel class"""

        # Initialize members of the class
        self.customer_identifier = customer_identifier
        self.account_identifier = account_identifier
        self.label = label
        self.ip_address = ip_address
        self.credit_card = credit_card
        self.billing_address = billing_address
        self.contact_information = contact_information


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        customer_identifier = dictionary.get('customerIdentifier')
        account_identifier = dictionary.get('accountIdentifier')
        label = dictionary.get('label')
        ip_address = dictionary.get('ipAddress')
        credit_card = raas.models.new_credit_card_model.NewCreditCardModel.from_dictionary(dictionary.get('creditCard')) if dictionary.get('creditCard') else None
        billing_address = raas.models.billing_address_model.BillingAddressModel.from_dictionary(dictionary.get('billingAddress')) if dictionary.get('billingAddress') else None
        contact_information = None
        if dictionary.get('contactInformation') != None:
            contact_information = list()
            for structure in dictionary.get('contactInformation'):
                contact_information.append(raas.models.full_name_email_model.FullNameEmailModel.from_dictionary(structure))

        # Return an object of this model
        return cls(customer_identifier,
                   account_identifier,
                   label,
                   ip_address,
                   credit_card,
                   billing_address,
                   contact_information)





# -*- coding: utf-8 -*-

"""
    raas.models.order_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper
import raas.models.currency_breakdown_model
import raas.models.reward_model
import raas.models.name_email_model

class OrderModel(object):

    """Implementation of the 'Order' model.

    Represents the response from a get order call

    Attributes:
        reference_order_id (string): The reference order id
        customer_identifier (string): The customer identifier
        account_identifier (string): The account identifier
        account_number (string): The account number
        amount_charged (CurrencyBreakdownModel): The order's amount
            information
        denomination (CurrencyBreakdownModel): Information about the gift card
            amount
        utid (string): The UTID
        reward_name (string): The reward name
        send_email (bool): Indicates if an email was sent to the recipient
        status (string): The order's status
        created_at (datetime): When the order was placed
        reward (RewardModel): Contains the reward credentials
        sender (NameEmailModel): The sender data
        recipient (NameEmailModel): The recipient data
        etid (string): The email template id
        campaign (string): An optional campaign identifier
        email_subject (string): The subject of the email
        external_ref_id (string): An external reference id
        message (string): A message included with the email
        notes (string): Optional customer notes
        margin_share (CurrencyBreakdownModel): Margin share information

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "reference_order_id":'referenceOrderID',
        "customer_identifier":'customerIdentifier',
        "account_identifier":'accountIdentifier',
        "account_number":'accountNumber',
        "amount_charged":'amountCharged',
        "denomination":'denomination',
        "utid":'utid',
        "reward_name":'rewardName',
        "send_email":'sendEmail',
        "status":'status',
        "created_at":'createdAt',
        "reward":'reward',
        "sender":'sender',
        "recipient":'recipient',
        "etid":'etid',
        "campaign":'campaign',
        "email_subject":'emailSubject',
        "external_ref_id":'externalRefID',
        "message":'message',
        "notes":'notes',
        "margin_share":'marginShare'
    }

    def __init__(self,
                 reference_order_id=None,
                 customer_identifier=None,
                 account_identifier=None,
                 account_number=None,
                 amount_charged=None,
                 denomination=None,
                 utid=None,
                 reward_name=None,
                 send_email=None,
                 status=None,
                 created_at=None,
                 reward=None,
                 sender=None,
                 recipient=None,
                 etid=None,
                 campaign=None,
                 email_subject=None,
                 external_ref_id=None,
                 message=None,
                 notes=None,
                 margin_share=None):
        """Constructor for the OrderModel class"""

        # Initialize members of the class
        self.reference_order_id = reference_order_id
        self.customer_identifier = customer_identifier
        self.account_identifier = account_identifier
        self.account_number = account_number
        self.amount_charged = amount_charged
        self.denomination = denomination
        self.utid = utid
        self.reward_name = reward_name
        self.send_email = send_email
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.reward = reward
        self.sender = sender
        self.recipient = recipient
        self.etid = etid
        self.campaign = campaign
        self.email_subject = email_subject
        self.external_ref_id = external_ref_id
        self.message = message
        self.notes = notes
        self.margin_share = margin_share


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        reference_order_id = dictionary.get('referenceOrderID')
        customer_identifier = dictionary.get('customerIdentifier')
        account_identifier = dictionary.get('accountIdentifier')
        account_number = dictionary.get('accountNumber')
        amount_charged = raas.models.currency_breakdown_model.CurrencyBreakdownModel.from_dictionary(dictionary.get('amountCharged')) if dictionary.get('amountCharged') else None
        denomination = raas.models.currency_breakdown_model.CurrencyBreakdownModel.from_dictionary(dictionary.get('denomination')) if dictionary.get('denomination') else None
        utid = dictionary.get('utid')
        reward_name = dictionary.get('rewardName')
        send_email = dictionary.get('sendEmail')
        status = dictionary.get('status')
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdAt")).datetime if dictionary.get("createdAt") else None
        reward = raas.models.reward_model.RewardModel.from_dictionary(dictionary.get('reward')) if dictionary.get('reward') else None
        sender = raas.models.name_email_model.NameEmailModel.from_dictionary(dictionary.get('sender')) if dictionary.get('sender') else None
        recipient = raas.models.name_email_model.NameEmailModel.from_dictionary(dictionary.get('recipient')) if dictionary.get('recipient') else None
        etid = dictionary.get('etid')
        campaign = dictionary.get('campaign')
        email_subject = dictionary.get('emailSubject')
        external_ref_id = dictionary.get('externalRefID')
        message = dictionary.get('message')
        notes = dictionary.get('notes')
        margin_share = raas.models.currency_breakdown_model.CurrencyBreakdownModel.from_dictionary(dictionary.get('marginShare')) if dictionary.get('marginShare') else None

        # Return an object of this model
        return cls(reference_order_id,
                   customer_identifier,
                   account_identifier,
                   account_number,
                   amount_charged,
                   denomination,
                   utid,
                   reward_name,
                   send_email,
                   status,
                   created_at,
                   reward,
                   sender,
                   recipient,
                   etid,
                   campaign,
                   email_subject,
                   external_ref_id,
                   message,
                   notes,
                   margin_share)





# -*- coding: utf-8 -*-

"""
    raas.models.raas_client_error_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class RaasClientErrorModel(object):

    """Implementation of the 'RaasClientError' model.

    Represents a RaaS 4xx Error

    Attributes:
        path (string): The path to the invalid value
        message (string): The error message
        constraint (string): The constraint validated
        invalid_value (string): The value that was invalid

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "path":'path',
        "message":'message',
        "constraint":'constraint',
        "invalid_value":'invalidValue'
    }

    def __init__(self,
                 path=None,
                 message=None,
                 constraint=None,
                 invalid_value=None):
        """Constructor for the RaasClientErrorModel class"""

        # Initialize members of the class
        self.path = path
        self.message = message
        self.constraint = constraint
        self.invalid_value = invalid_value


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        path = dictionary.get('path')
        message = dictionary.get('message')
        constraint = dictionary.get('constraint')
        invalid_value = dictionary.get('invalidValue')

        # Return an object of this model
        return cls(path,
                   message,
                   constraint,
                   invalid_value)





# -*- coding: utf-8 -*-

"""
    raas.models.resend_order_response_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class ResendOrderResponseModel(object):

    """Implementation of the 'ResendOrderResponse' model.

    Represents the response returned from a resend order request

    Attributes:
        created_at (datetime): When the resend request was created
        id (int): The order resend id

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "created_at":'createdAt',
        "id":'id'
    }

    def __init__(self,
                 created_at=None,
                 id=None):
        """Constructor for the ResendOrderResponseModel class"""

        # Initialize members of the class
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.id = id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdAt")).datetime if dictionary.get("createdAt") else None
        id = dictionary.get('id')

        # Return an object of this model
        return cls(created_at,
                   id)





# -*- coding: utf-8 -*-

"""
    raas.models.create_account_request_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateAccountRequestModel(object):

    """Implementation of the 'CreateAccountRequest' model.

    Represents the request to create an account

    Attributes:
        account_identifier (string): The account identifier
        display_name (string): The display name
        contact_email (string): The contact email

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "account_identifier":'accountIdentifier',
        "display_name":'displayName',
        "contact_email":'contactEmail'
    }

    def __init__(self,
                 account_identifier=None,
                 display_name=None,
                 contact_email=None):
        """Constructor for the CreateAccountRequestModel class"""

        # Initialize members of the class
        self.account_identifier = account_identifier
        self.display_name = display_name
        self.contact_email = contact_email


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        account_identifier = dictionary.get('accountIdentifier')
        display_name = dictionary.get('displayName')
        contact_email = dictionary.get('contactEmail')

        # Return an object of this model
        return cls(account_identifier,
                   display_name,
                   contact_email)





# -*- coding: utf-8 -*-

"""
    raas.models.get_orders_response_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.page_model
import raas.models.order_summary_model

class GetOrdersResponseModel(object):

    """Implementation of the 'GetOrdersResponse' model.

    Represents the response from the get orders call

    Attributes:
        page (PageModel): Pagination information
        orders (list of OrderSummaryModel): An array of orders

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "page":'page',
        "orders":'orders'
    }

    def __init__(self,
                 page=None,
                 orders=None):
        """Constructor for the GetOrdersResponseModel class"""

        # Initialize members of the class
        self.page = page
        self.orders = orders


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        page = raas.models.page_model.PageModel.from_dictionary(dictionary.get('page')) if dictionary.get('page') else None
        orders = None
        if dictionary.get('orders') != None:
            orders = list()
            for structure in dictionary.get('orders'):
                orders.append(raas.models.order_summary_model.OrderSummaryModel.from_dictionary(structure))

        # Return an object of this model
        return cls(page,
                   orders)





# -*- coding: utf-8 -*-

"""
    tests.http_response_catcher

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

from raas.http.http_call_back import HttpCallBack

class HttpResponseCatcher(HttpCallBack):

    """A class used for catching the HttpResponse object from controllers.
    
    This class inherits HttpCallBack and implements the on_after_response
    method to catch the HttpResponse object as returned by the HttpClient
    after a request is executed.

    """
    def on_before_request(self, request):
        pass;

    def on_after_response(self, context):
        self.response = context.response






# -*- coding: utf-8 -*-

"""
    raas.models.credit_card_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper
import raas.models.full_name_email_model

class CreditCardModel(object):

    """Implementation of the 'CreditCard' model.

    Represents a Credit Card

    Attributes:
        customer_identifier (string): The customer identifier
        account_identifier (string): The account identifier
        token (string): The credit card token
        label (string): The label/nickname for the credit card
        last_four_digits (string): The last four digits of the credit card
            number
        expiration_date (string): The credit card's expiration date
        status (string): The status of the credit card
        created_date (datetime): The date the card was added
        activation_date (datetime): The date the card will be available for
            use
        contact_information (list of FullNameEmailModel): An optional array of
            FullNameEmail objects

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "customer_identifier":'customerIdentifier',
        "account_identifier":'accountIdentifier',
        "token":'token',
        "label":'label',
        "last_four_digits":'lastFourDigits',
        "expiration_date":'expirationDate',
        "status":'status',
        "created_date":'createdDate',
        "activation_date":'activationDate',
        "contact_information":'contactInformation'
    }

    def __init__(self,
                 customer_identifier=None,
                 account_identifier=None,
                 token=None,
                 label=None,
                 last_four_digits=None,
                 expiration_date=None,
                 status=None,
                 created_date=None,
                 activation_date=None,
                 contact_information=None):
        """Constructor for the CreditCardModel class"""

        # Initialize members of the class
        self.customer_identifier = customer_identifier
        self.account_identifier = account_identifier
        self.token = token
        self.label = label
        self.last_four_digits = last_four_digits
        self.expiration_date = expiration_date
        self.status = status
        self.created_date = APIHelper.RFC3339DateTime(created_date) if created_date else None
        self.activation_date = APIHelper.RFC3339DateTime(activation_date) if activation_date else None
        self.contact_information = contact_information


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        customer_identifier = dictionary.get('customerIdentifier')
        account_identifier = dictionary.get('accountIdentifier')
        token = dictionary.get('token')
        label = dictionary.get('label')
        last_four_digits = dictionary.get('lastFourDigits')
        expiration_date = dictionary.get('expirationDate')
        status = dictionary.get('status')
        created_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdDate")).datetime if dictionary.get("createdDate") else None
        activation_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("activationDate")).datetime if dictionary.get("activationDate") else None
        contact_information = None
        if dictionary.get('contactInformation') != None:
            contact_information = list()
            for structure in dictionary.get('contactInformation'):
                contact_information.append(raas.models.full_name_email_model.FullNameEmailModel.from_dictionary(structure))

        # Return an object of this model
        return cls(customer_identifier,
                   account_identifier,
                   token,
                   label,
                   last_four_digits,
                   expiration_date,
                   status,
                   created_date,
                   activation_date,
                   contact_information)





# -*- coding: utf-8 -*-

"""
    raas.controllers.catalog_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.catalog_model import CatalogModel
from ..exceptions.raas_generic_exception import RaasGenericException

class CatalogController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(CatalogController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def get_catalog(self):
        """Does a GET request to /catalogs.

        Retrieves a platform's catalog

        Returns:
            CatalogModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_catalog called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_catalog.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/catalogs'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_catalog.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_catalog.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_catalog')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_catalog.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, CatalogModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    tests.controllers.test_status_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper


class StatusControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(StatusControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.status

    # Tests if we can retrieve the system status
    def test_test_status_is_ok(self):

        # Perform the API call through the SDK function
        result = self.controller.get_system_status()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize('{"status":"UP"}')
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))





# -*- coding: utf-8 -*-

"""
    raas.models.billing_address_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class BillingAddressModel(object):

    """Implementation of the 'BillingAddress' model.

    Represents a Billing Address

    Attributes:
        first_name (string): The first name
        last_name (string): The last name
        address_line_1 (string): The address
        city (string): The city
        state (string): The state/province
        postal_code (string): The postal code
        country (string): The 2-letter country code
        email_address (string): The billing contact's email address
        address_line_2 (string): An optional second address line

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "first_name":'firstName',
        "last_name":'lastName',
        "address_line_1":'addressLine1',
        "city":'city',
        "state":'state',
        "postal_code":'postalCode',
        "country":'country',
        "email_address":'emailAddress',
        "address_line_2":'addressLine2'
    }

    def __init__(self,
                 first_name=None,
                 last_name=None,
                 address_line_1=None,
                 city=None,
                 state=None,
                 postal_code=None,
                 country=None,
                 email_address=None,
                 address_line_2=None):
        """Constructor for the BillingAddressModel class"""

        # Initialize members of the class
        self.first_name = first_name
        self.last_name = last_name
        self.address_line_1 = address_line_1
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.country = country
        self.email_address = email_address
        self.address_line_2 = address_line_2


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        first_name = dictionary.get('firstName')
        last_name = dictionary.get('lastName')
        address_line_1 = dictionary.get('addressLine1')
        city = dictionary.get('city')
        state = dictionary.get('state')
        postal_code = dictionary.get('postalCode')
        country = dictionary.get('country')
        email_address = dictionary.get('emailAddress')
        address_line_2 = dictionary.get('addressLine2')

        # Return an object of this model
        return cls(first_name,
                   last_name,
                   address_line_1,
                   city,
                   state,
                   postal_code,
                   country,
                   email_address,
                   address_line_2)





# -*- coding: utf-8 -*-

"""
    raas.models.account_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class AccountModel(object):

    """Implementation of the 'Account' model.

    Represents an Account

    Attributes:
        account_identifier (string): An account identifier
        account_number (string): An account number
        display_name (string): A display name
        currency_code (string): The currency code for the account
        current_balance (float): The current balance of the account
        created_at (datetime): The date the account was created
        status (string): The status of the account
        contact_email (string): The contact email on file for the account

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "account_identifier":'accountIdentifier',
        "account_number":'accountNumber',
        "display_name":'displayName',
        "currency_code":'currencyCode',
        "current_balance":'currentBalance',
        "created_at":'createdAt',
        "status":'status',
        "contact_email":'contactEmail'
    }

    def __init__(self,
                 account_identifier=None,
                 account_number=None,
                 display_name=None,
                 currency_code='USD',
                 current_balance=None,
                 created_at=None,
                 status=None,
                 contact_email=None):
        """Constructor for the AccountModel class"""

        # Initialize members of the class
        self.account_identifier = account_identifier
        self.account_number = account_number
        self.display_name = display_name
        self.currency_code = currency_code
        self.current_balance = current_balance
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.status = status
        self.contact_email = contact_email


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        account_identifier = dictionary.get('accountIdentifier')
        account_number = dictionary.get('accountNumber')
        display_name = dictionary.get('displayName')
        currency_code = dictionary.get("currencyCode") if dictionary.get("currencyCode") else 'USD'
        current_balance = dictionary.get('currentBalance')
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdAt")).datetime if dictionary.get("createdAt") else None
        status = dictionary.get('status')
        contact_email = dictionary.get('contactEmail')

        # Return an object of this model
        return cls(account_identifier,
                   account_number,
                   display_name,
                   currency_code,
                   current_balance,
                   created_at,
                   status,
                   contact_email)





# -*- coding: utf-8 -*-

"""
    raas.raas_client

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""
from .decorators import lazy_property
from .configuration import Configuration
from .controllers.status_controller import StatusController
from .controllers.accounts_controller import AccountsController
from .controllers.orders_controller import OrdersController
from .controllers.fund_controller import FundController
from .controllers.exchange_rates_controller import ExchangeRatesController
from .controllers.customers_controller import CustomersController
from .controllers.catalog_controller import CatalogController

class RaasClient(object):

    config = Configuration

    @lazy_property
    def status(self):
        return StatusController()

    @lazy_property
    def accounts(self):
        return AccountsController()

    @lazy_property
    def orders(self):
        return OrdersController()

    @lazy_property
    def fund(self):
        return FundController()

    @lazy_property
    def exchange_rates(self):
        return ExchangeRatesController()

    @lazy_property
    def customers(self):
        return CustomersController()

    @lazy_property
    def catalog(self):
        return CatalogController()


    def __init__(self, 
                 platform_name = 'QAPlatform2',
                 platform_key = 'apYPfT6HNONpDRUj3CLGWYt7gvIHONpDRUYPfT6Hj'):
        if platform_name != None:
            Configuration.platform_name = platform_name
        if platform_key != None:
            Configuration.platform_key = platform_key





# -*- coding: utf-8 -*-

"""
    raas.models.exchange_rate_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class ExchangeRateModel(object):

    """Implementation of the 'ExchangeRate' model.

    Represents an Exchange Rate

    Attributes:
        last_modified_date (datetime): When the exchange rate was last
            updated
        reward_currency (string): The reward currency
        base_currency (string): The base currency
        base_fx (float): The exchange rate

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "last_modified_date":'lastModifiedDate',
        "reward_currency":'rewardCurrency',
        "base_currency":'baseCurrency',
        "base_fx":'baseFx'
    }

    def __init__(self,
                 last_modified_date=None,
                 reward_currency=None,
                 base_currency=None,
                 base_fx=None):
        """Constructor for the ExchangeRateModel class"""

        # Initialize members of the class
        self.last_modified_date = APIHelper.RFC3339DateTime(last_modified_date) if last_modified_date else None
        self.reward_currency = reward_currency
        self.base_currency = base_currency
        self.base_fx = base_fx


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        last_modified_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("lastModifiedDate")).datetime if dictionary.get("lastModifiedDate") else None
        reward_currency = dictionary.get('rewardCurrency')
        base_currency = dictionary.get('baseCurrency')
        base_fx = dictionary.get('baseFx')

        # Return an object of this model
        return cls(last_modified_date,
                   reward_currency,
                   base_currency,
                   base_fx)





# -*- coding: utf-8 -*-

"""
    raas.controllers.exchange_rates_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.exchange_rate_response_model import ExchangeRateResponseModel
from ..exceptions.raas_generic_exception import RaasGenericException

class ExchangeRatesController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(ExchangeRatesController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def get_exchange_rates(self):
        """Does a GET request to /exchangerates.

        Retrieve current exchange rates

        Returns:
            ExchangeRateResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_exchange_rates called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_exchange_rates.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/exchangerates'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_exchange_rates.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_exchange_rates.')
            _request = self.http_client.get(_query_url, headers=_headers)
            BasicAuth.apply(_request)
            _context = self.execute_request(_request, name = 'get_exchange_rates')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_exchange_rates.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, ExchangeRateResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    raas.models.full_name_email_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class FullNameEmailModel(object):

    """Implementation of the 'FullNameEmail' model.

    Represents a full name and an email address

    Attributes:
        full_name (string): The full name
        email_address (string): The email address

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "full_name":'fullName',
        "email_address":'emailAddress'
    }

    def __init__(self,
                 full_name=None,
                 email_address=None):
        """Constructor for the FullNameEmailModel class"""

        # Initialize members of the class
        self.full_name = full_name
        self.email_address = email_address


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        full_name = dictionary.get('fullName')
        email_address = dictionary.get('emailAddress')

        # Return an object of this model
        return cls(full_name,
                   email_address)





# -*- coding: utf-8 -*-

"""
    raas.models.customer_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper
import raas.models.account_summary_model

class CustomerModel(object):

    """Implementation of the 'Customer' model.

    Represents a Customer/Group

    Attributes:
        customer_identifier (string): The customer identifier
        display_name (string): The display name
        status (string): The status of the customer
        created_at (datetime): The date the customer was created
        accounts (list of AccountSummaryModel): An array of AccountSummary
            objects

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "customer_identifier":'customerIdentifier',
        "display_name":'displayName',
        "status":'status',
        "created_at":'createdAt',
        "accounts":'accounts'
    }

    def __init__(self,
                 customer_identifier=None,
                 display_name=None,
                 status=None,
                 created_at=None,
                 accounts=None):
        """Constructor for the CustomerModel class"""

        # Initialize members of the class
        self.customer_identifier = customer_identifier
        self.display_name = display_name
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.accounts = accounts


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        customer_identifier = dictionary.get('customerIdentifier')
        display_name = dictionary.get('displayName')
        status = dictionary.get('status')
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdAt")).datetime if dictionary.get("createdAt") else None
        accounts = None
        if dictionary.get('accounts') != None:
            accounts = list()
            for structure in dictionary.get('accounts'):
                accounts.append(raas.models.account_summary_model.AccountSummaryModel.from_dictionary(structure))

        # Return an object of this model
        return cls(customer_identifier,
                   display_name,
                   status,
                   created_at,
                   accounts)





__all__ = [
    'base_controller',
    'status_controller',
    'accounts_controller',
    'orders_controller',
    'fund_controller',
    'exchange_rates_controller',
    'customers_controller',
    'catalog_controller',
]


# -*- coding: utf-8 -*-

"""
    raas.models.page_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class PageModel(object):

    """Implementation of the 'Page' model.

    Represents the pagination information returned in a paginated API call

    Attributes:
        number (int): The page number
        elements_per_block (int): The number of elements per page
        result_count (int): The number of results returned
        total_count (int): The total number of results

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "number":'number',
        "elements_per_block":'elementsPerBlock',
        "result_count":'resultCount',
        "total_count":'totalCount'
    }

    def __init__(self,
                 number=None,
                 elements_per_block=None,
                 result_count=None,
                 total_count=None):
        """Constructor for the PageModel class"""

        # Initialize members of the class
        self.number = number
        self.elements_per_block = elements_per_block
        self.result_count = result_count
        self.total_count = total_count


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        number = dictionary.get('number')
        elements_per_block = dictionary.get('elementsPerBlock')
        result_count = dictionary.get('resultCount')
        total_count = dictionary.get('totalCount')

        # Return an object of this model
        return cls(number,
                   elements_per_block,
                   result_count,
                   total_count)





# -*- coding: utf-8 -*-

"""
    raas.models.reward_credential_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class RewardCredentialModel(object):

    """Implementation of the 'RewardCredential' model.

    Represents a Reward Credential

    Attributes:
        label (string): The label of the credential
        value (string): The actual credential
        mtype (string): The type of the credential

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "label":'label',
        "value":'value',
        "mtype":'type'
    }

    def __init__(self,
                 label=None,
                 value=None,
                 mtype=None):
        """Constructor for the RewardCredentialModel class"""

        # Initialize members of the class
        self.label = label
        self.value = value
        self.mtype = mtype


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        label = dictionary.get('label')
        value = dictionary.get('value')
        mtype = dictionary.get('type')

        # Return an object of this model
        return cls(label,
                   value,
                   mtype)





# -*- coding: utf-8 -*-

"""
    raas.models.item_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class ItemModel(object):

    """Implementation of the 'Item' model.

    Represents an item

    Attributes:
        utid (string): The UTID
        reward_name (string): The reward name
        currency_code (string): The currency code
        status (string): The item's status
        value_type (string): The item's value type (VARIABLE_VALUE or
            FIXED_VALUE)
        reward_type (string): The reward type
        created_date (datetime): The date the item was created
        last_update_date (datetime): The date the item was last updated
        countries (list of string): The countries this item is valid in
        min_value (float): The minimum orderable value (for variable value
            items)
        max_value (float): The maximum orderable value (for variable value
            items)
        face_value (float): The face value of the gift card

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "utid":'utid',
        "reward_name":'rewardName',
        "currency_code":'currencyCode',
        "status":'status',
        "value_type":'valueType',
        "reward_type":'rewardType',
        "created_date":'createdDate',
        "last_update_date":'lastUpdateDate',
        "countries":'countries',
        "min_value":'minValue',
        "max_value":'maxValue',
        "face_value":'faceValue'
    }

    def __init__(self,
                 utid=None,
                 reward_name=None,
                 currency_code=None,
                 status=None,
                 value_type=None,
                 reward_type=None,
                 created_date=None,
                 last_update_date=None,
                 countries=None,
                 min_value=None,
                 max_value=None,
                 face_value=None):
        """Constructor for the ItemModel class"""

        # Initialize members of the class
        self.utid = utid
        self.reward_name = reward_name
        self.currency_code = currency_code
        self.status = status
        self.value_type = value_type
        self.reward_type = reward_type
        self.created_date = APIHelper.RFC3339DateTime(created_date) if created_date else None
        self.last_update_date = APIHelper.RFC3339DateTime(last_update_date) if last_update_date else None
        self.countries = countries
        self.min_value = min_value
        self.max_value = max_value
        self.face_value = face_value


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        utid = dictionary.get('utid')
        reward_name = dictionary.get('rewardName')
        currency_code = dictionary.get('currencyCode')
        status = dictionary.get('status')
        value_type = dictionary.get('valueType')
        reward_type = dictionary.get('rewardType')
        created_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdDate")).datetime if dictionary.get("createdDate") else None
        last_update_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("lastUpdateDate")).datetime if dictionary.get("lastUpdateDate") else None
        countries = dictionary.get('countries')
        min_value = dictionary.get('minValue')
        max_value = dictionary.get('maxValue')
        face_value = dictionary.get('faceValue')

        # Return an object of this model
        return cls(utid,
                   reward_name,
                   currency_code,
                   status,
                   value_type,
                   reward_type,
                   created_date,
                   last_update_date,
                   countries,
                   min_value,
                   max_value,
                   face_value)





# -*- coding: utf-8 -*-

"""
    raas.models.reward_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
import raas.models.reward_credential_model

class RewardModel(object):

    """Implementation of the 'Reward' model.

    Represents a Reward

    Attributes:
        credentials (dict<object, string>): A map of reward credentials
        credential_list (list of RewardCredentialModel): An array of reward
            credentials
        redemption_instructions (string): Reward redemption instructions

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "credentials":'credentials',
        "credential_list":'credentialList',
        "redemption_instructions":'redemptionInstructions'
    }

    def __init__(self,
                 credentials=None,
                 credential_list=None,
                 redemption_instructions=None):
        """Constructor for the RewardModel class"""

        # Initialize members of the class
        self.credentials = credentials
        self.credential_list = credential_list
        self.redemption_instructions = redemption_instructions


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        credentials = dictionary.get('credentials')
        credential_list = None
        if dictionary.get('credentialList') != None:
            credential_list = list()
            for structure in dictionary.get('credentialList'):
                credential_list.append(raas.models.reward_credential_model.RewardCredentialModel.from_dictionary(structure))
        redemption_instructions = dictionary.get('redemptionInstructions')

        # Return an object of this model
        return cls(credentials,
                   credential_list,
                   redemption_instructions)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_orders_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper
from raas.models.create_order_request_model import CreateOrderRequestModel


class OrdersControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(OrdersControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.orders

    # Tests retrieving a single order
    def test_test_get_order(self):
        # Parameters for the API call
        reference_order_id = 'RA180426-1401-64'

        # Perform the API call through the SDK function
        result = self.controller.get_order(reference_order_id)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"referenceOrderID":"RA180426-1401-64","customerIdentifier":"sdkautotest4"'
            ',"accountIdentifier":"sdkautotest5","accountNumber":"A32386768","amountChar'
            'ged":{"value":1,"currencyCode":"USD","total":1},"marginShare":{"value":0,"c'
            'urrencyCode":"USD"},"denomination":{"value":1,"currencyCode":"USD"},"utid":'
            '"U561593","rewardName":"Reward Link","sender":{"firstName":"","lastName":""'
            ',"email":""},"recipient":{"email":"","firstName":"","lastName":""},"sendEma'
            'il":false,"status":"COMPLETE","createdAt":"2018-04-26T20:08:59.624Z","rewar'
            'd":{"credentials":{"Redemption URL":"https://sandbox.rewardlink.io/r/1/ed0H'
            'tzSblNV6oFddNnnlf68eXzGQoREvcxwxu_Vi5Wk"},"credentialList":[{"label":"Redem'
            'ption URL","value":"https://sandbox.rewardlink.io/r/1/ed0HtzSblNV6oFddNnnlf'
            '68eXzGQoREvcxwxu_Vi5Wk","type":"url"}],"redemptionInstructions":"<p>&bull; '
            'Click on the redemption link above to activate your Reward Link.<br />\\r'
            '\\n&bull; Next, you will be able to spend your balance on retail gift cards'
            '.</p>\\r\\n\\r\\n<p>If you don&#39;t want to spend your entire Reward Link '
            'value right away, save the email or URL and return via the redemption link '
            '- your value will be waiting to be spent. This allows you to store the bala'
            'nce to redeem at another time.</p>\\r\\n"}}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body, check_values = True))


    # Tests placing an order with the minimum parameters required
    def test_test_place_order_with_minimum_criteria(self):
        # Parameters for the API call
        body = APIHelper.json_deserialize(
            '{"accountIdentifier":"sdkautotest5","amount":1.00,"customerIdentifier":"sd'
            'kautotest4","sendEmail":false,"utid":"U561593"}'
            , CreateOrderRequestModel.from_dictionary)

        # Perform the API call through the SDK function
        result = self.controller.create_order(body)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 201)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"referenceOrderID":"RA180426-1401-64","customerIdentifier":"sdkautotest4"'
            ',"accountIdentifier":"sdkautotest5","accountNumber":"A32386768","amountChar'
            'ged":{"value":1,"currencyCode":"USD","total":1},"marginShare":{"value":0,"c'
            'urrencyCode":"USD"},"denomination":{"value":1,"currencyCode":"USD"},"utid":'
            '"U561593","rewardName":"Reward Link","sender":{"firstName":"","lastName":""'
            ',"email":""},"recipient":{"email":"","firstName":"","lastName":""},"sendEma'
            'il":false,"status":"COMPLETE","createdAt":"2018-04-26T20:08:59.624Z","rewar'
            'd":{"credentials":{"Redemption URL":"https://sandbox.rewardlink.io/r/1/ed0H'
            'tzSblNV6oFddNnnlf68eXzGQoREvcxwxu_Vi5Wk"},"credentialList":[{"label":"Redem'
            'ption URL","value":"https://sandbox.rewardlink.io/r/1/ed0HtzSblNV6oFddNnnlf'
            '68eXzGQoREvcxwxu_Vi5Wk","type":"url"}],"redemptionInstructions":"<p>&bull; '
            'Click on the redemption link above to activate your Reward Link.<br />\\r'
            '\\n&bull; Next, you will be able to spend your balance on retail gift cards'
            '.</p>\\r\\n\\r\\n<p>If you don&#39;t want to spend your entire Reward Link '
            'value right away, save the email or URL and return via the redemption link '
            '- your value will be waiting to be spent. This allows you to store the bala'
            'nce to redeem at another time.</p>\\r\\n"}}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))





# -*- coding: utf-8 -*-

"""
    raas.controllers.status_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import logging
from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..models.system_status_response_model import SystemStatusResponseModel
from ..exceptions.raas_generic_exception import RaasGenericException

class StatusController(BaseController):

    """A Controller to access Endpoints in the raas API."""

    def __init__(self, client=None, call_back=None):
        super(StatusController, self).__init__(client, call_back)
        self.logger = logging.getLogger(__name__)

    def get_system_status(self):
        """Does a GET request to /pulse.

        Retrieves system status

        Returns:
            SystemStatusResponseModel: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        try:
            self.logger.info('get_system_status called.')
    
            # Prepare query URL
            self.logger.info('Preparing query URL for get_system_status.')
            _query_builder = Configuration.get_base_uri()
            _query_builder += '/pulse'
            _query_url = APIHelper.clean_url(_query_builder)
    
            # Prepare headers
            self.logger.info('Preparing headers for get_system_status.')
            _headers = {
                'accept': 'application/json'
            }
    
            # Prepare and execute request
            self.logger.info('Preparing and executing request for get_system_status.')
            _request = self.http_client.get(_query_url, headers=_headers)
            _context = self.execute_request(_request, name = 'get_system_status')

            # Endpoint and global error handling using HTTP status codes.
            self.logger.info('Validating response for get_system_status.')
            if _context.response.status_code == 0:
                raise RaasGenericException('API Error', _context)
            self.validate_response(_context)
    
            # Return appropriate type
            return APIHelper.json_deserialize(_context.response.raw_body, SystemStatusResponseModel.from_dictionary)

        except Exception as e:
            self.logger.error(e, exc_info = True)
            raise



# -*- coding: utf-8 -*-

"""
    raas.models.deposit_response_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""
from raas.api_helper import APIHelper

class DepositResponseModel(object):

    """Implementation of the 'DepositResponse' model.

    Represents the response from a place deposit call

    Attributes:
        amount (float): The funding amount
        amount_charged (float): The amount charged, after fees
        created_date (datetime): The date the transaction was created
        fee_percent (float): The processing fee percentage
        reference_deposit_id (string): The deposit reference id
        status (string): The status of the transaction

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount":'amount',
        "amount_charged":'amountCharged',
        "created_date":'createdDate',
        "fee_percent":'feePercent',
        "reference_deposit_id":'referenceDepositID',
        "status":'status'
    }

    def __init__(self,
                 amount=None,
                 amount_charged=None,
                 created_date=None,
                 fee_percent=None,
                 reference_deposit_id=None,
                 status=None):
        """Constructor for the DepositResponseModel class"""

        # Initialize members of the class
        self.amount = amount
        self.amount_charged = amount_charged
        self.created_date = APIHelper.RFC3339DateTime(created_date) if created_date else None
        self.fee_percent = fee_percent
        self.reference_deposit_id = reference_deposit_id
        self.status = status


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = dictionary.get('amount')
        amount_charged = dictionary.get('amountCharged')
        created_date = APIHelper.RFC3339DateTime.from_value(dictionary.get("createdDate")).datetime if dictionary.get("createdDate") else None
        fee_percent = dictionary.get('feePercent')
        reference_deposit_id = dictionary.get('referenceDepositID')
        status = dictionary.get('status')

        # Return an object of this model
        return cls(amount,
                   amount_charged,
                   created_date,
                   fee_percent,
                   reference_deposit_id,
                   status)





from setuptools import setup, find_packages

# Try to convert markdown README to rst format for PyPI.
try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(
    name='raas',
    version='2.0.0',
    description='With this RESTful API you can integrate a global reward or incentive program into your app or platform. If you have any questions or if you\'d like to receive your own credentials, please contact us at devsupport@tangocard.com.',
    long_description=long_description,
    author='APIMatic SDK Generator',
    author_email='support@apimatic.io',
    url='https://apimatic.io/',
    packages=find_packages(),
    install_requires=[
        'requests>=2.9.1, <3.0',
        'jsonpickle>=0.7.1, <1.0',
        'cachecontrol>=0.11.7, <1.0',
        'python-dateutil>=2.5.3, <3.0'
    ],
    tests_require=[
        'nose>=1.3.7'
    ],
    test_suite = 'nose.collector'
)


# -*- coding: utf-8 -*-

"""
    raas.models.raas_server_error_model

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io )
"""


class RaasServerErrorModel(object):

    """Implementation of the 'RaasServerError' model.

    Represents a RaaS 5xx Error

    Attributes:
        message (string): The error message
        code (int): The RaaS error code

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "message":'message',
        "code":'code'
    }

    def __init__(self,
                 message=None,
                 code=None):
        """Constructor for the RaasServerErrorModel class"""

        # Initialize members of the class
        self.message = message
        self.code = code


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        message = dictionary.get('message')
        code = dictionary.get('code')

        # Return an object of this model
        return cls(message,
                   code)





# -*- coding: utf-8 -*-

"""
    tests.controllers.test_exchange_rates_controller

    This file was automatically generated for Tango Card, Inc. by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from raas.api_helper import APIHelper


class ExchangeRatesControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(ExchangeRatesControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.exchange_rates

    # Tests if we can successfully retrieve exchange rates
    def test_test_retrieve_exchange_rates(self):

        # Perform the API call through the SDK function
        result = self.controller.get_exchange_rates()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)
        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"disclaimer":"Exchange rates are provided here for reference. They are up'
            'dated at least once a day and may have changed by time of order.","exchange'
            'Rates":[{"lastModifiedDate":"2018-04-19T13:00:14.291","rewardCurrency":"EUR'
            '","baseCurrency":"NZD","baseFx":0.59120},{"lastModifiedDate":"2018-04-19T13'
            ':00:14.291","rewardCurrency":"USD","baseCurrency":"NZD","baseFx":0.73130},{'
            '"lastModifiedDate":"2018-04-19T13:00:14.230","rewardCurrency":"INR","baseCu'
            'rrency":"AUD","baseFx":51.29066}]}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))





__all__ = [
    'create_account_request_model',
    'name_email_model',
    'system_status_response_model',
    'raas_server_error_model',
    'new_credit_card_model',
    'create_customer_request_model',
    'unregister_credit_card_response_model',
    'unregister_credit_card_request_model',
    'create_order_request_model',
    'deposit_request_model',
    'get_orders_response_model',
    'item_model',
    'catalog_model',
    'reward_credential_model',
    'order_model',
    'exchange_rate_model',
    'full_name_email_model',
    'raas_client_error_model',
    'brand_model',
    'billing_address_model',
    'account_model',
    'create_credit_card_request_model',
    'get_deposit_response_model',
    'reward_model',
    'currency_breakdown_model',
    'account_summary_model',
    'resend_order_response_model',
    'deposit_response_model',
    'exchange_rate_response_model',
    'customer_model',
    'order_summary_model',
    'page_model',
    'credit_card_model',
]